import 'package:diameter/components/detail.dart';
import 'package:diameter/components/dialogs.dart';
import 'package:diameter/components/dropdown.dart';
import 'package:diameter/components/forms.dart';
import 'package:diameter/config.dart';
import 'package:diameter/models/accuracy.dart';
import 'package:diameter/models/meal_category.dart';
import 'package:diameter/models/meal_portion_type.dart';
import 'package:diameter/models/meal_source.dart';
import 'package:diameter/navigation.dart';
import 'package:flutter/material.dart';

class MealSourceDetailScreen extends StatefulWidget {
  static const String routeName = '/meal-source';

  final int id;

  const MealSourceDetailScreen({Key? key, this.id = 0}) : super(key: key);

  @override
  _MealSourceDetailScreenState createState() => _MealSourceDetailScreenState();
}

class _MealSourceDetailScreenState extends State<MealSourceDetailScreen> {
  MealSource? _mealSource;
  bool _isNew = true;

  List<Accuracy> _portionSizeAccuracies = [];
  List<Accuracy> _carbsRatioAccuracies = [];
  List<MealCategory> _mealCategories = [];
  List<MealPortionType> _mealPortionTypes = [];

  final GlobalKey<FormState> _mealSourceForm = GlobalKey<FormState>();

  final _valueController = TextEditingController(text: '');
  final _notesController = TextEditingController(text: '');
  Accuracy? _defaultCarbsRatioAccuracy;
  Accuracy? _defaultPortionSizeAccuracy;
  MealCategory? _defaultMealCategory;
  MealPortionType? _defaultMealPortionType;

  @override
  void initState() {
    super.initState();

    reload();

    _portionSizeAccuracies = Accuracy.getAllForPortionSize();
    _carbsRatioAccuracies = Accuracy.getAllForCarbsRatio();
    _mealCategories = MealCategory.getAll();
    _mealPortionTypes = MealPortionType.getAll();

    if (_mealSource != null) {
      _valueController.text = _mealSource!.value;
      _notesController.text = _mealSource!.notes ?? '';

      _defaultPortionSizeAccuracy =
          _mealSource!.defaultPortionSizeAccuracy.target;
      _defaultCarbsRatioAccuracy =
          _mealSource!.defaultCarbsRatioAccuracy.target;

      _defaultMealCategory = _mealSource!.defaultMealCategory.target;
      _defaultMealPortionType =
          _mealSource!.defaultMealPortionType.target;
    }
  }

  void reload() {
    if (widget.id != 0) {
      setState(() {
        _mealSource = MealSource.get(widget.id);
      });
    }
    _isNew = _mealSource == null;
  }

  void handleSaveAction() async {
    MealSource mealSource = MealSource(
        id: widget.id,
        value: _valueController.text,
        notes: _notesController.text,
      );
      mealSource.defaultCarbsRatioAccuracy.target = _defaultCarbsRatioAccuracy;
      mealSource.defaultPortionSizeAccuracy.target =
          _defaultPortionSizeAccuracy;
      mealSource.defaultMealCategory.target = _defaultMealCategory;
      mealSource.defaultMealPortionType.target = _defaultMealPortionType;
      MealSource.put(mealSource);
      Navigator.pop(context, '${_isNew ? 'New' : ''} Meal Source saved');
  }

  void handleCancelAction() {
    if (showConfirmationDialogOnCancel &&
        ((_isNew &&
                (_valueController.text != '' ||
                    _defaultCarbsRatioAccuracy != null ||
                    _defaultPortionSizeAccuracy != null ||
                    _defaultMealCategory != null ||
                    _defaultMealPortionType != null ||
                    _notesController.text != '')) ||
            (!_isNew &&
                (_valueController.text != _mealSource!.value ||
                    _defaultCarbsRatioAccuracy !=
                        _mealSource!.defaultCarbsRatioAccuracy.target ||
                    _defaultPortionSizeAccuracy !=
                        _mealSource!.defaultPortionSizeAccuracy.target ||
                    _defaultMealCategory !=
                        _mealSource!.defaultMealCategory.target ||
                    _defaultMealPortionType !=
                        _mealSource!.defaultMealPortionType.target ||
                    _notesController.text !=
                        (_mealSource!.notes ?? ''))))) {
      Dialogs.showCancelConfirmationDialog(
        context: context,
        isNew: _isNew,
        onSave: handleSaveAction,
      );
    } else {
      Navigator.pop(context);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(_isNew ? 'New Meal Source' : _mealSource!.value),
      ),
      drawer:
          const Navigation(currentLocation: MealSourceDetailScreen.routeName),
      body: SingleChildScrollView(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: <Widget>[
            FormWrapper(
              formState: _mealSourceForm,
              fields: [
                TextFormField(
                  controller: _valueController,
                  decoration: const InputDecoration(
                    labelText: 'Name',
                  ),
                  validator: (value) {
                    if (value!.trim().isEmpty) {
                      return 'Empty name';
                    }
                    return null;
                  },
                ),
                AutoCompleteDropdownButton<Accuracy>(
                  selectedItem: _defaultCarbsRatioAccuracy,
                  label: 'Default Carbs Ratio Accuracy',
                  items: _carbsRatioAccuracies,
                  renderItem: (item) => item.value,
                  onChanged: (value) {
                    setState(() {
                      _defaultCarbsRatioAccuracy = value;
                    });
                  },
                ),
                AutoCompleteDropdownButton<Accuracy>(
                  selectedItem: _defaultPortionSizeAccuracy,
                  label: 'Default Portion Size Accuracy',
                  items: _portionSizeAccuracies,
                  renderItem: (item) => item.value,
                  onChanged: (value) {
                    setState(() {
                      _defaultPortionSizeAccuracy = value;
                    });
                  },
                ),
                AutoCompleteDropdownButton<MealCategory>(
                  selectedItem: _defaultMealCategory,
                  label: 'Default Meal Category',
                  items: _mealCategories,
                  renderItem: (item) => item.value,
                  onChanged: (value) {
                    setState(() {
                      _defaultMealCategory = value;
                    });
                  },
                ),
                AutoCompleteDropdownButton<MealPortionType>(
                  selectedItem: _defaultMealPortionType,
                  label: 'Default Meal Portion Type',
                  items: _mealPortionTypes,
                  renderItem: (item) => item.value,
                  onChanged: (value) {
                    setState(() {
                      _defaultMealPortionType = value;
                    });
                  },
                ),
                TextFormField(
                  controller: _notesController,
                  decoration: const InputDecoration(
                    labelText: 'Notes',
                    alignLabelWithHint: true,
                  ),
                  keyboardType: TextInputType.multiline,
                )
              ],
            ),
          ],
        ),
      ),
      bottomNavigationBar: DetailBottomRow(
        onCancel: handleCancelAction,
        onSave: handleSaveAction,
      ),
    );
  }
}
